@@ -302,6 +302,7 @@ class Generator:
         needed a _generateDescription for whereAmI. :-) See below.
         """
         result = []
+        self._script.pointOfReference['usedDescriptionForName'] = False
         name = self._script.utilities.displayedText(obj)
         if obj.getRole() == pyatspi.ROLE_COMBO_BOX:
             children = self._script.utilities.selectedChildren(obj)
@@ -413,6 +414,12 @@ class Generator:
             desc = obj.description.lower()
             if not (desc in name.lower() or desc in label.lower()):
                 result.append(obj.description)
+
+        if not result:
+            desc = self._script.utilities.displayedDescription(obj)
+            if desc:
+                result.append(desc)
+
         return result
 
     def _generateLabel(self, obj, **args):
