@@ -27,8 +27,10 @@ __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc." \
                 "Copyright (c) 2011-2016 Igalia, S.L."
 __license__   = "LGPL"
 
+import math
 import pyatspi
 import time
+from gi.repository import Gdk
 from gi.repository import GLib
 
 from . import debug
@@ -66,6 +68,11 @@ class KeyboardEvent(InputEvent):
     duplicateCount = 0
     orcaModifierPressed = False
 
+    # Whether last press of the Orca modifier was alone
+    lastOrcaModifierAlone = False
+    # Whether the current press of the Orca modifier is alone
+    currentOrcaModifierAlone = False
+
     TYPE_UNKNOWN          = "unknown"
     TYPE_PRINTABLE        = "printable"
     TYPE_MODIFIER         = "modifier"
@@ -107,6 +114,7 @@ class KeyboardEvent(InputEvent):
         self._consume_reason = None
         self._did_consume = None
         self._result_reason = None
+        self._bypassOrca = None
 
         if self._script:
             self._script.checkKeyboardEventData(self)
@@ -143,6 +151,12 @@ class KeyboardEvent(InputEvent):
             role = None
         _mayEcho = _isPressed or role == pyatspi.ROLE_TERMINAL
 
+        if not self.isOrcaModifier():
+            if KeyboardEvent.orcaModifierPressed:
+                KeyboardEvent.currentOrcaModifierAlone = False
+            else:
+                KeyboardEvent.lastOrcaModifierAlone = False
+
         if self.isNavigationKey():
             self.keyType = KeyboardEvent.TYPE_NAVIGATION
             self.shouldEcho = _mayEcho and settings.enableNavigationKeys
@@ -153,7 +167,17 @@ class KeyboardEvent(InputEvent):
             self.keyType = KeyboardEvent.TYPE_MODIFIER
             self.shouldEcho = _mayEcho and settings.enableModifierKeys
             if self.isOrcaModifier():
-                KeyboardEvent.orcaModifierPressed = _isPressed
+                if KeyboardEvent.lastOrcaModifierAlone:
+                    # double-orca, let the real action happen
+                    self._bypassOrca = True
+                    if not _isPressed:
+                        KeyboardEvent.lastOrcaModifierAlone = False
+                else:
+                    KeyboardEvent.orcaModifierPressed = _isPressed
+                    if _isPressed:
+                        KeyboardEvent.currentOrcaModifierAlone = True
+                    else:
+                        KeyboardEvent.lastOrcaModifierAlone = KeyboardEvent.currentOrcaModifierAlone
         elif self.isFunctionKey():
             self.keyType = KeyboardEvent.TYPE_FUNCTION
             self.shouldEcho = _mayEcho and settings.enableFunctionKeys
@@ -317,7 +341,7 @@ class KeyboardEvent(InputEvent):
         if not self.event_string in lockingKeys:
             return False
 
-        if not orca_state.bypassNextCommand:
+        if not orca_state.bypassNextCommand and not self._bypassOrca:
             return not self.event_string in settings.orcaModifierKeys
 
         return True
@@ -610,6 +634,14 @@ class KeyboardEvent(InputEvent):
     def _process(self):
         """Processes this input event."""
 
+        if self._bypassOrca:
+            if self.event_string == "Caps_Lock" \
+               and self.type == pyatspi.KEY_PRESSED_EVENT:
+                    self._lock_mod()
+                    self.keyType = KeyboardEvent.TYPE_LOCKING
+                    self._present()
+            return False, 'Bypassed orca modifier'
+
         orca_state.lastInputEvent = self
         if not self.isModifierKey():
             orca_state.lastNonModifierKeyEvent = self
@@ -648,6 +680,27 @@ class KeyboardEvent(InputEvent):
 
         return False, 'Unaddressed case'
 
+    def _lock_mod(self):
+        def lock_mod(modifiers):
+            def lockit():
+                try:
+                    modifier = (1 << pyatspi.MODIFIER_SHIFTLOCK)
+                    if modifiers & modifier:
+                        lock = pyatspi.KEY_UNLOCKMODIFIERS
+                        debug.println(debug.LEVEL_INFO, "Locking capslock", True)
+                    else:
+                        lock = pyatspi.KEY_LOCKMODIFIERS
+                        debug.println(debug.LEVEL_INFO, "Unlocking capslock", True)
+                    pyatspi.Registry.generateKeyboardEvent(modifier, None, lock)
+                    debug.println(debug.LEVEL_INFO, "Done with capslock", True)
+                except:
+                    debug.println(debug.LEVEL_INFO, "Could not trigger capslock, " \
+                        "at-spi2-core >= 2.30 is needed for triggering capslock", True)
+                    pass
+            return lockit
+        debug.println(debug.LEVEL_INFO, "Scheduling capslock", True)
+        GLib.timeout_add(1, lock_mod(self.modifiers))
+
     def _consume(self):
         startTime = time.time()
         data = "'%s' (%d)" % (self.event_string, self.hw_code)
@@ -680,6 +733,13 @@ class BrailleEvent(InputEvent):
 
 class MouseButtonEvent(InputEvent):
 
+    try:
+        display = Gdk.Display.get_default()
+        seat = Gdk.Display.get_default_seat(display)
+        _pointer = seat.get_pointer()
+    except:
+        _pointer = None
+
     def __init__(self, event):
         """Creates a new InputEvent of type MOUSE_BUTTON_EVENT."""
 
@@ -688,6 +748,35 @@ class MouseButtonEvent(InputEvent):
         self.y = event.detail2
         self.pressed = event.type.endswith('p')
         self.button = event.type[len("mouse:button:"):-1]
+        self._script = orca_state.activeScript
+        self.window = None
+        self.obj = None
+
+        if self.pressed:
+            self._validateCoordinates()
+
+        if not self._script:
+            return
+
+        self.window = self._script.utilities.activeWindow()
+        if not self.window:
+            return
+
+        self.obj = self._script.utilities.descendantAtPoint(
+            self.window, self.x, self.y, event.any_data)
+
+    def _validateCoordinates(self):
+        if not self._pointer:
+            return
+
+        screen, x, y = self._pointer.get_position()
+        if math.sqrt((self.x - x)**2 + (self.y - y)**2) < 25:
+            return
+
+        msg = "WARNING: Event coordinates (%i, %i) may be bogus. " \
+              "Updating to (%i, %i)." % (self.x, self.y, x, y)
+        debug.println(debug.LEVEL_INFO, msg, True)
+        self.x, self.y = x, y
 
     def setClickCount(self):
         """Updates the count of the number of clicks a user has made."""
@@ -695,6 +784,7 @@ class MouseButtonEvent(InputEvent):
         if not self.pressed:
             return
 
+        lastInputEvent = orca_state.lastInputEvent
         if not isinstance(lastInputEvent, MouseButtonEvent):
             self._clickCount = 1
             return
