@@ -290,14 +290,6 @@ class Script(default.Script):
         prefs.update(self.spellcheck.getPreferencesFromGUI())
         return prefs
 
-    def doWhereAmI(self, inputEvent, basicOnly):
-        """Performs the whereAmI operation."""
-
-        if self.spellcheck.isActive():
-            self.spellcheck.presentErrorDetails(not basicOnly)
-
-        super().doWhereAmI(inputEvent, basicOnly)
-
     def presentObject(self, obj, **args):
         if not self._lastCommandWasStructNav:
             super().presentObject(obj, **args)
@@ -435,28 +427,6 @@ class Script(default.Script):
 
         return True
 
-    def columnConvert(self, column):
-        """ Convert a spreadsheet column into it's column label
-
-        Arguments:
-        - column: the column number to convert.
-
-        Returns a string representing the spread sheet column.
-        """
-
-        base26 = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-
-        if column <= len(base26):
-            return base26[column-1]
-
-        res = ""
-        while column > 0:
-            digit = column % len(base26)
-            res = " " + base26[digit-1] + res
-            column /= len(base26)
-
-        return res
-
     def setDynamicRowHeaders(self, inputEvent):
         """Set the column for the dynamic header rows to use when speaking
         calc cell entries. In order to set the column, the user should first
@@ -478,7 +448,7 @@ class Script(default.Script):
         if table:
             self.dynamicRowHeaders[hash(table)] = column
             self.presentMessage(
-                messages.DYNAMIC_ROW_HEADER_SET % self.columnConvert(column+1))
+                messages.DYNAMIC_ROW_HEADER_SET % self.utilities.columnConvert(column+1))
 
         return True
 
@@ -824,6 +794,11 @@ class Script(default.Script):
             default.Script.onCheckedChanged(self, event)
             return
  
+        sourceWindow = self.utilities.topLevelObject(obj)
+        focusWindow = self.utilities.topLevelObject(orca_state.locusOfFocus)
+        if sourceWindow != focusWindow:
+            return
+
         # Announce when the toolbar buttons are toggled if we just toggled
         # them; not if we navigated to some text.
         weToggledIt = False
@@ -866,6 +841,14 @@ class Script(default.Script):
     def onSelectionChanged(self, event):
         """Callback for object:selection-changed accessibility events."""
 
+        if self.utilities.isSpreadSheetTable(event.source):
+            if _settingsManager.getSetting('onlySpeakDisplayedText'):
+                return
+            if self.utilities.handleRowAndColumnSelectionChange(event.source):
+                return
+            self.utilities.handleCellSelectionChange(event.source)
+            return
+
         if not self.utilities.isComboBoxSelectionChange(event):
             super().onSelectionChanged(event)
             return
@@ -882,6 +865,10 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             return
 
+        if event.source != orca_state.locusOfFocus \
+           and event.source.getState().contains(pyatspi.STATE_FOCUSED):
+            orca.setLocusOfFocus(event, event.source, True)
+
         super().onTextSelectionChanged(event)
 
     def getTextLineAtCaret(self, obj, offset=None, startOffset=None, endOffset=None):
